from django import forms
from django.contrib.auth.models import Group, Permission, User
from django.contrib.auth import password_validation
from django.utils.text import slugify
from django.utils.translation import ugettext as _
from thorson_wiki.models import *

class ArticleUpdateForm(forms.ModelForm):

    def __init__(self, *args, **kwargs):

        super(ArticleUpdateForm, self).__init__(*args, **kwargs)

        users = User.objects.all()
        groups = Group.objects.all()

        user_choices = [(user.username, user) for user in users]
        group_choices = [(group.name, group) for group in groups]

        choices = [
            (_("Groups"), group_choices),
            (_("Users"), user_choices),
        ]

        self.fields['read_permissions'] = forms.MultipleChoiceField(
            label=_("Read permissions"),
            choices=choices,
            required=False,
            help_text=_("Select users and groups to grant permission" \
                    " to read this article.")
        )

        self.fields['edit_permissions'] = forms.MultipleChoiceField(
            label=_("Edit permissions"),
            choices=choices,
            required=False,
            help_text=_("Select users and groups to grant permission" \
                    " to edit this article.")
        )

    def save(self, commit=True):
        """
        If commit is set to False, then permissions cannot be updated. Be sure
        to call :func:`save_permissions` at some point if commit is set to
        False.
        """

        article = super(ArticleUpdateForm, self).save(commit=commit)

        if commit:
            self.save_permissions(article)

        return article

    def save_permissions(self, article):
        """
        Saves an article's permissions.
        """

        read_permissions = self.cleaned_data['read_permissions']
        read_permission = Permission.objects.get(
            codename='read-article:%s' % article.slug
        )
        for item in read_permissions:
            if isinstance(item, User):
                item.user_permissions.add(read_permission)
            elif isinstance(item, Group):
                item.permissions.add(read_permission)

        edit_permissions = self.cleaned_data['edit_permissions']
        edit_permission = Permission.objects.get(
            codename='edit-article:%s' % article.slug
        )
        for item in read_permissions:
            if isinstance(item, User):
                item.user_permissions.add(edit_permission)
            elif isinstance(item, Group):
                item.permissions.add(edit_permission)

    def clean_title(self):

        title = self.cleaned_data.get('title')

        if self.instance:
            objects = Article.objects.exclude(id=self.instance.id)
        else:
            objects = Article.objects.all()

        if objects.filter(slug=slugify(title)).exists():
            raise forms.ValidationError(
                    _("An article with that name already" \
                            " exists."),
                    code='already_exists'
            )

        return title

    class Meta:

        model = Article
        fields = ['title', 'namespace', 'content', 'public',
                'redirect']

class UserCreateForm(forms.ModelForm):

    error_messages = {
        'password_mismatch': _("The two password fields didn't match."),
    }

    password1 = forms.CharField(label=_("Password"),
            widget=forms.PasswordInput)
    password2 = forms.CharField(label=_("Password confirmation"),
            widget=forms.PasswordInput,
            help_text=_("Enter your password again."))

    def clean_password2(self):

        password1 = self.cleaned_data.get('password1')
        password2 = self.cleaned_data.get('password2')

        if password1 and password2 == password1:
            pass
        else:
            raise forms.ValidationError(
                    self.error_messages['password_mismatch'],
                    code='password_mismatch',
            )

        password_validation.validate_password(
                self.cleaned_data.get('password2')
        )

        return password2

    def save(self, commit=True):

        user = super(UserCreateForm, self).save(commit=commit)
        user.username = self.cleaned_data.get('username')
        user.set_password(self.cleaned_data['password1'])

        if commit:
            user.save()

        return user

    class Meta:
        model = User
        fields = ['username', 'password1', 'password2', 'email']

class UserUpdateForm(forms.ModelForm):

    class Meta:

        model = User
        fields = ['username', 'email']



import re

from django.core.urlresolvers import reverse
from django.utils.text import slugify
from markdown.extensions import Extension
from markdown.inlinepatterns import Pattern
from markdown.util import etree

def build_url(label, namespace_):
    """
    Build a URL from the label and namespace. The base and end are
    automatically added by Django.
    """

    if ':' in label:
        namespace_, label = label.split(':', 1)

    return reverse(namespace_ + ':article',
            kwargs={'slug': slugify(label)})

class ThorsonWikiLinkExtension(Extension):

    ARTICLE_NAME = r'(?P<article_name>[\w\s\-\'/,:]+?)'

    LINK_TEXT = r'(?:\|(?P<link_text>[\w\s\-\'/.]+))?'

    WIKILINK_RE = '\[\[' + ARTICLE_NAME + LINK_TEXT + '\]\]'

    def __init__(self, *args, **kwargs):
        self.config = {
            'base_url': ['/', 'String to append to beginning or URL.'],
            'end_url': ['/', 'String to append to end of URL.'],
            'html_class': ['wikilink', 'CSS hook. Leave blank for none.'],
            'build_url': [build_url, 'Callable formats URL from label.'],
        }

        super(ThorsonWikiLinkExtension, self).__init__(*args, **kwargs)

    def extendMarkdown(self, md, md_globals):
        self.md = md

        # append to end of inline patterns
        wikilinkPattern = WikiLinks(self.WIKILINK_RE, self.getConfigs())
        wikilinkPattern.md = md
        md.inlinePatterns.add('wikilink', wikilinkPattern, "<not_strong")

class WikiLinks(Pattern):
    def __init__(self, pattern, config):
        super(WikiLinks, self).__init__(pattern)
        self.config = config

    def handleMatch(self, m):
        namespace_ = getattr(self.md, 'namespace_', '')

        wikilink = m.group('article_name')
        link_text = m.group('link_text')

        if wikilink.strip():
            base_url, end_url, html_class = self._getMeta()
            label = m.group(2).strip()
            url = self.config['build_url'](label, namespace_)
            a = etree.Element('a')

            if link_text:
                a.text = link_text
            else:
                a.text = label
            a.set('href', url)
            if html_class:
                a.set('class', html_class)
        else:
            a = ''
        return a

    def _getMeta(self):
        """ Return meta data or config data. """
        base_url = self.config['base_url']
        end_url = self.config['end_url']
        html_class = self.config['html_class']
        if hasattr(self.md, 'Meta'):
            if 'wiki_base_url' in self.md.Meta:
                base_url = self.md.Meta['wiki_base_url'][0]
            if 'wiki_end_url' in self.md.Meta:
                end_url = self.md.Meta['wiki_end_url'][0]
            if 'wiki_html_class' in self.md.Meta:
                html_class = self.md.Meta['wiki_html_class'][0]
        return base_url, end_url, html_class

def makeExtension(*args, **kwargs):
    return ThorsonWikiLinkExtension(*args, **kwargs)



import bleach

from django import template
from django.conf import settings
from django.core.urlresolvers import reverse
from django.utils.module_loading import import_string
from django.utils.safestring import mark_safe
from django.utils.translation import ugettext as _
from markdown.extensions import Extension as BaseExtension
from thorson_wiki.markup import ThorsonMarkdown

register = template.Library()

@register.filter
def can_read(user, article):
    """
    Returns True if the given user has the necessary permissions to read the
    given article.
    """

    return article.can_read(user)

@register.filter
def can_edit(user, article):
    """
    Returns True if the given user has the necessary permissions to read the
    given article.
    """

    return article.can_edit(user)

@register.tag(name='markdown')
def md2html(parser, token):

    content = parser.parse(('endmarkdown',))
    parser.delete_first_token()

    return MarkdownNode(content)

class MarkdownNode(template.Node):

    def __init__(self, content):

        self.content = content

    def render(self, context):

        namespace = context.get('namespace', None)
        namespace_ = context.get('namespace_', None)
        request = context.get('request', None)

        extensions = namespace.get('md_extensions', [])
        configs = namespace.get('md_configs', {})

        md = ThorsonMarkdown(
            extensions=extensions,
            extension_configs=configs,
            namespace=namespace,
            namespace_=namespace_,
            request=request
        )

        content = self.content.render(context).strip()

        allowed_tags = namespace.get('allowed_tags', bleach.ALLOWED_TAGS)
        allowed_attrs = namespace.get('allowed_attrs',
                bleach.ALLOWED_ATTRIBUTES)
        html = md.convert(content)

        return mark_safe(bleach.clean(html, tags=allowed_tags,
            attributes=allowed_attrs))
